"""``face`` contains the code for rendering the face.

Most of the code in this package is not exposed to the user; however,
the user can access the FacePresets file with some pre-defined
face configurations to set the appearance of the face.

.. autosummary::
    :toctree:

    pylips.face.FacePresets

"""

from pylips.face import FacePresets