"""A simple library for web-based conversational agents."""

__author__ = "Interaction Lab"
__email__ = "dennler@usc.edu"
__version__ = "0.0.10"

from pylips import face, speech

__all__ = [
    "speech",
    "face",
]